<?php
/*
	Copyright 2010 Ketil Tunheim. This file is part of Magic Fretboard.

    Magic Fretboard is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Magic Fretboard is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

function set_session() {
	global $notes_back, $notes_rev, $intervals_all, $octave;

	// remember the old root
	$old_root = $_SESSION['root'];
	$display = $_SESSION['display_type'];
	$temperament = $_SESSION['temperament'];

	// unset old session
	$_SESSION = array();
	activate_ints();
	
	// get the list of new note names
	$new_scale = explode(' ', $_REQUEST['scale']);
	$new_tuning = explode(' ', $_REQUEST['tuning']);
	
	// how many?
	$_SESSION['strings'] = count($new_tuning);
	$_SESSION['scale_steps'] = count($new_scale);
	
	// set them to the new session
	foreach ($new_tuning as $i => $notename) {
		$_SESSION[TU.$i] = $notes_rev[$notename];
	}
	foreach($new_scale as $i => $notename) {
		$_SESSION[ST.$i] = $notes_back[$notename];
	}
	// uncheck any boxes if asked
	foreach ($intervals_all as $int) {
		if ($_REQUEST[IN.$int] == 'off') {
			$_SESSION[IN.$int] = 0;
			//echo 'off';
		}
	}
	
	// set the display option
	$_SESSION['display_type'] = ($_REQUEST['display']) ? $_REQUEST['display'] : $display;
	// new root is based on the first note in the new scale
	$root_mayhap = $_SESSION[ST.'0'];
	if ($root_mayhap < $_SESSION[TU.'0']) {
		$above_zero = $_SESSION[TU.'0'] % $octave;
		$root_mayhap += $above_zero*$octave;
	}
	$_SESSION['root'] = $root_mayhap;
	// restore temperament
	$_SESSION['temperament'] = $temperament;
}

function skew_session (&$info) {
	global $octave, $notes_all;

	// gather the old scale into an array
	$scale = array();
	for ($i = 0; $i < $_SESSION['scale_steps']; $i++) {
		$scale[$i] = $_SESSION[ST.$i] % $octave;
	}
	// here's some necessary old data
	$scale_steps = $_SESSION['scale_steps'];
	$old_root	= $_SESSION['root'];
	
	// and here's the new data
	$new_root	= $_REQUEST['newroot'];
	$type		= $_REQUEST['type'];

	// don't reset the session: all fields will be overwritten
	if ($type == 'enharmonic') activate_ints();
	$_SESSION['type'] = $type;
	
	// now, check if the new root is a valid interval to the old
	$interval = $new_root % $octave;
	$root_pos = array_search($interval,$scale);
	
	if ($type == 'enharmonic' && $root_pos === false) {
		// that's no good
		$info = '<br style="clear:both" />That isn\'t one of the current positions. You\'ll have to do a normal transposition first.';
	} else {
		// okay, move the scale
		$_SESSION['root'] = $new_root;
		$diff = $new_root - $old_root;
		
		if ($type == 'chromatic') {
			// recalculate the locations
			for ($i = 0; $i < $scale_steps; $i++) {
				$step = ($scale[$i] + $diff)%$octave;
				while ($step < 0) {
					$step += $octave;
				}
				$_SESSION[ST.$i] = $step;
			}
		} else {
			// rotate the values of the scale array
			foreach ($scale as $key => $interva) {				
				if ($key < $root_pos) {
					// if it's not the root, put it on the end
					$_SESSION[ST.($scale_steps-1-$key)] = $interva;
				} else {
					// add right away
					$_SESSION[ST.($key-$root_pos)] = $interva;
				}
			} // end foreach
		} // end if
	} // end if
} // end function

function reset_session() {
	global $scale_def, $tuning_def, $root_def, $type_def, $display_def, $temperament_def;

	// reset session
	$_SESSION = array();
	activate_ints();
	
	// set to standard
	for ($i = 0; $i < count($scale_def); $i++) {
		$_SESSION[ST.$i] = $scale_def[$i];
	}
	for ($i = 0; $i < count($tuning_def); $i++) {
		$_SESSION[TU.$i] = $tuning_def[$i];
	}
	$_SESSION['root'] = $root_def;
	$_SESSION['scale_steps'] = count($scale_def);
	$_SESSION['strings'] = count($tuning_def);
	$_SESSION['type_type'] = $type_def;
	$_SESSION['display_type'] = $display_def;
	$_SESSION['temperament'] = $temperament_def;
}

function generate_string($string_num,$scale,$tuning,$root,$interv,$display) {
	global $notes_all, $max_fret, $octave;
	$string_row = '';

	// getting the image location of the open string
	$string_note = substr($notes_all[$tuning[$string_num]],0,-1);
	$string_img = strtolower($string_note) . '.gif';
	$string_img = str_replace(chr(35),'s',$string_img);
	
	// that'll be the first in the row of images
	$string_row .= '<img src="fretboard/' . $string_img . '" border="0" />'.NL;
	
	// now for all the frets
	for ($fret = 0; $fret <= $max_fret; $fret++) {
		$string_row .= get_position($string_num,$fret,$scale,$tuning,$root,$interv,$display);
		
		// start of fretboard or change of octave?
		if ($fret == 0) {
			$string_row .= '<img src="fretboard/open.gif" border="0" />'.NL;
		} else if (($fret+1)%$octave == 0) {
			$string_row .= '<img src="fretboard/thick.gif" border="0" />'.NL;
		}
	}
	$string_row .= '<br />'.NL;
	
	return $string_row;	
}

function get_position($string_num,$fret,$scale,$tuning,$root,$interv,$display) {
	global $intervals_all, $notes_all;
	$img_code = '';

	// the note number of this position
	$note_num = $tuning[$string_num]+$fret;
	$from_root = $note_num % 12;
	
	// is it in the current scale?
	$match = array_search($from_root,$scale);
	if ($match !== false) {
		// which interval is this in relation to the root?
		$interval = ($tuning[$string_num] + $fret - $root%12) % 12;
		
		// shall I display universal key, or notes?
		if ($display == 'notes') {
			$file = substr($notes_all[$from_root],0,-1);
			$file = strtolower($file);
			$file = str_replace(chr(35),'s',$file);
		} else {
			$file = $intervals_all[$interval];
			$file = str_replace(chr(35),'s',$file);
		}
		
		// and with the checkboxes as they are, that makes the postfix what?
		if ($interv[$interval] == 0) {
			$pf = '_dim';
		} else if ($display == 'notes') {
			$pf = '_f';
		} else {
			$pf = '';
		}
		
		// image file
		$img = 'fretboard/' . $file . $pf . '.gif';
	} else {
		// pass the new root if you click in the wrong place, too!
		$img = 'fretboard/empty.gif';
	}
	// get the name of the note (no longer necessary)
	//$name = $notes_all[$string_num + $fret];
	
	// when clicked, pass the number of the new root
	$img_code .= '<img src="' . $img .'" border="0" onClick="sendForm('.$note_num.')" />'.$nl;
	
	return $img_code;
}

function get_tables($checked,$available) {
	global $intervals_all, $temperament, $notes_all, $root, $octave;
	
	$sql_part = '';
	foreach ($intervals_all as $key => $value) {
		// internal part for getting the 'optional' parameter
		$part_part = "IF(ai.optional = 1,1,0)";
		// this is going to be a long query
		$sql_part .= ", SUM(IF(it." . $temperament . "=" . $key . ",".$part_part.",NULL)) AS '" . $key . "'";
	}
	
	$sql = "SELECT at.navn AS navn, at.navn_long " . $sql_part . "
			FROM ".PF."akkordtype at, ".PF."akkordtype_innhold ai, ".PF."intervall it
			WHERE ai.akkordtype_id = at.id
			AND it.id = ai.interval_id
			GROUP BY at.navn ORDER BY at.id ASC";
	$result = mysql_query($sql) or die("Chord type query failed: " . mysql_error());

	// is this chord possible?
	while ($data = mysql_fetch_array($result)) {
		$having = array();
		$having_zero = array();
		$using = array();
		
		foreach ($intervals_all as $key => $value) {
			// does this chord have this interval?
			if ($data[$key] != null) {
				$having[$key] = $value;
				// is it NOT optional?
				if ($data[$key] == 0) {
					$having_zero[$key] = $value;
					// is it checked off and in this scale?
					if ($checked[$key%12] == 1 && array_key_exists($key%$octave,$available)) {
						$using[$key] = $value;
					}
				}
			}
		}
		
		if (array_keys($using) == array_keys($having_zero)) {
			$chord = substr($notes_all[$root],0,-1) . $data['navn'];
			$rows .= '<tr>'.NL.'<td>' . $chord . '</td>'.NL.'<td>( 1';
			foreach ($having as $key => $value) {
				if (!array_key_exists($key,$using)) {
					$rows .= ' <u>' . $value . '</u> ';
				} else {
					$rows .= ' ' . $value . ' ';
				}
			}
			$rows .= ')</td>'.NL.'</tr>'.NL;
		}
	}
	return $rows;
}

function activate_ints() {
	global $intervals_all;

	foreach ($intervals_all as $int) {
		$_SESSION[IN.$int] = 1;
	}
}

function uncheck_ints() {
	global $intervals_all;
	
	// did something get unchecked?
	foreach($intervals_all as $ints) {
		if ($_REQUEST[$ints] == 'on') {
			$_SESSION[IN.$ints] = 1;
		} else {
			$_SESSION[IN.$ints] = 0;
		}
	}
	
	// or did the display options change?
	$_SESSION['display_type'] = $_REQUEST['disp_form'];
}

?>